/***************************************
    This is part of frox: A simple transparent FTP proxy
    Copyright (C) 2000 James Hollingshead

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    common.h -- miscellaneous definitions.
					
***************************************/


#ifndef COMMON_H
#define COMMON_H    /*+ To stop multiple inclusions. +*/

#include "../config.h"

#include <stdio.h>
#include <sys/types.h>

#if STDC_HEADERS
# include <stdlib.h>
# include <string.h>
#elif HAVE_STRINGS_H
# include <strings.h>
#endif /* STDC_HEADERS */

#if HAVE_UNISTD_H
# include <unistd.h>
#endif

#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif /* HAVE_SYS_TIME_H */
#endif /* TIME_WITH_SYS_TIME */

#include "systwrap.h"

/* Defines */
#ifdef DEBUG
#define debug(S) fprintf(stderr,S)
#define debug2(S,T) fprintf(stderr,S,T)
#else
#define debug(S)
#define debug2(S,T)
#endif

#define TRUE 1
#define FALSE 0

#define DM    242
#define IP    244
#define WILL  251
#define WONT  252
#define DO    253
#define DONT  254
#define IAC   255

#define BUF_LEN 4097
#define MAX_LINE_LEN 180  /* For control connections */

/* States - this is a bitfield. active/passive, whether we are listening 
on any ports, and whether we are connected to data ports on either client
or server.
 -------------------------------------------
| listen | clientdata | serverdata | active |
 -------------------------------------------
*/
#define ACTIVE            1
#define SERVERDATA        2
#define CLIENTDATA        4
#define CONNECTED         6
#define LISTEN            8

/* Error codes */
typedef enum _exitreason {
	TIMEOUT,          
	CLIENT_CLOSE,
	SERVER_CLOSE,      
	NO_CLIENT_DATA,       /* Unable to connect to it */
	NO_SERVER_DATA,    
	CLIENT_RUBBISH,       /* Malformed control stream */
	SERVER_RUBBISH,
	MALLOC
} exitreason;    

typedef enum _loglevel {
	INFO,                 /* Who connects, etc.*/
	ERROR,                /* Non critical errors*/
	ATTACK                /* Suspicious stuff */
} loglevel;

/* Data Structures */
typedef struct _address_info {
	u_int32_t addr;
	u_int16_t port;
} address_info;

typedef struct _connection {
	address_info address;
	int fd;
	unsigned char buf[BUF_LEN];
	int buflen;
} connection;

typedef struct _session_info {
	connection client_control;
	connection server_control;
	connection client_data;
	connection server_data;

	int state;
	int listen;
	int anon;
} session_info;

/*****************
**   misc.c   **
******************/
int listen_on_socket(address_info *listen_address);
int connect_to_socket(address_info listen_address);
address_info com2n(int a1,int a2,int a3,int a4,int p1,int p2);
void n2com(address_info address, int* a1,int* a2,int* a3,int* a4,
	   int* p1,int* p2);
void closeandexit(int status);
address_info extract_address(const unsigned char *buf);
void write_log(loglevel priority, unsigned char * msg, ...);
unsigned char *addr2name(u_int32_t address);
void quit(exitreason reason);
void do_linecontrol(unsigned char *buf);

/*****************
**   config.c   **
******************/
int process_cmdline(int argc, char *argv[]);
int read_config(void);
u_int32_t config_getaddress(void);
u_int16_t config_getport(void);
int config_maxforks(void);
int config_usesameaddress(void);
int config_bouncedefend(void);
int config_bindtodevice(int fd);
int config_chroot(void);
int config_droppriv(void);
int config_usecache(void);
const char* config_ftpproxy(void);
struct timeval config_timeout(void);

/*******************
**Global Variables**
********************/ 
session_info *info;

#endif /* COMMON_H */
